#!/bin/sh
#
# Nagios plugin to check Solaris 10 patch status with PCA.
#
# $Id: check_solaris_pca.sh,v 1.4 2008/09/09 07:46:17 kivimaki Exp $
#
# Copyright (C) 2008  Hannu Kivimki / CSC - Scientific Computing Ltd.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

# ------------------------------ SETTINGS --------------------------------------

# External commands
CMD_AWK="/usr/bin/awk"
CMD_PCA="/usr/sbin/pca"

# Nagios plugin return values
STATE_OK=0
STATE_WARNING=1
STATE_CRITICAL=2
STATE_UNKNOWN=3
STATE_TEXT=""
PERF_TEXT=""

# Default values
FLAG_R=""
FLAG_S=""
FLAG=B=""
CHECK_OTHER=0
CHECK_ALL=1
VERBOSE=0

#--------------------------- PRE-FLIGHT CHECKS ---------------------------------

if [ ! -x $CMD_PCA ]; then
    echo "Error: pca command ($CMD_PCA) not found or not executable."
    exit $STATE_UNKNOWN
fi

# ------------------------------ FUNCTIONS -------------------------------------

printInfo() {
    echo "Nagios plugin to check Solaris 10 patch status with PCA."
    echo "Copyright (C) 2008  Hannu Kivimki / CSC - Scientific Computing Ltd."
}

printHelp() {
    echo
    echo "Usage: `basename $0` [-r][-s][-b][-o] -w <limit> -c <limit>"
    echo
    echo "  -r  count recommended patches (R flag on)"
    echo "  -s  count security patches (S flag on)"
    echo "  -b  count bad patches (B flag on)"
    echo "  -o  count other patches (no flags)"
    echo "  -w  warning threshold (number of patches)"
    echo "  -c  critical threshold (number of patches)"
    echo "  -h  this help screen"
    echo "  -l  license info"
    echo "  -v  verbose output (for debugging)"
    echo "  -V  version info"
    echo
    echo "If multiple patch types are given, they are OR'ed together:"
    echo " -rs  = count patches with R or S flags or both"
    echo " -rso = count patches with R or S flags or both or none"
    echo "        (but not patches with B flag)"
    echo
    echo "Without any patch type options plugin will default to all (-rsbo)."
    echo
    echo "Example: `basename $0` -rs -w 10 -c 20"
    echo
    echo "This will return CRITICAL if 20 or more patches with R or S"
    echo "flags (or both) are available, WARNING if there are 10 or more"
    echo "patches and OK otherwise."
    echo
}

printLicense() {
    echo
    echo "This program is free software; you can redistribute it and/or"
    echo "modify it under the terms of the GNU General Public License"
    echo "as published by the Free Software Foundation; either version 2"
    echo "of the License, or (at your option) any later version."
    echo
    echo "This program is distributed in the hope that it will be useful,"
    echo "but WITHOUT ANY WARRANTY; without even the implied warranty of"
    echo "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
    echo "GNU General Public License for more details."
    echo
    echo "You should have received a copy of the GNU General Public License"
    echo "along with this program; if not, write to the Free Software"
    echo "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA."
    echo
}

printVersion() {
    echo
    echo "\$Id: check_solaris_pca.sh,v 1.4 2008/09/09 07:46:17 kivimaki Exp $"
    echo
}

# Checks command line options (pass $@ as parameter).
checkOptions() {
    if [ $# -eq 0 ]; then
        printInfo
        printHelp
        exit $STATE_UNKNOWN
    fi

    opt_recommended=0
    opt_security=0
    opt_bad=0
    while getopts rsbow:c:lhvV OPT $@; do
            case $OPT in
                r) opt_recommended=1
                   FLAG_R="R"
                   ;;
                s) opt_security=1
                   FLAG_S="S"
                   ;;
                b) opt_bad=1
                   FLAG_B="B"
                   ;;
                o) CHECK_OTHER=1
                   ;;
                w) # warning limit
                   opt_warn_limit=$OPTARG
                   ;;
                c) # critical limit
                   opt_crit_limit=$OPTARG
                   ;;
                l) printInfo
                   printLicense
                   exit $STATE_UNKNOWN
                   ;;
                h) printInfo
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
                v) VERBOSE=1
                   ;;
                V) printInfo
                   printVersion
                   exit $STATE_UNKNOWN
                   ;;
                ?) printInfo
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
            esac
    done

    limit_error=0
    if [ "`echo $opt_warn_limit | grep '^[0-9]*\$'`" = "" ]; then
        limit_error=1
    elif [ "`echo $opt_crit_limit | grep '^[0-9]*\$'`" = "" ]; then
        limit_error=1
    elif [ $opt_warn_limit -gt $opt_crit_limit ]; then
        limit_error=1
    fi
    if [ $limit_error -eq 1 ]; then
        echo "Error: Invalid threshold values."
        printInfo
        printHelp
        exit $STATE_UNKNOWN
    fi
    PATCH_WARN_LIMIT=$opt_warn_limit
    PATCH_CRIT_LIMIT=$opt_crit_limit

    # Check whether we are supposed to check all patches or just some types.
    #
    case `expr $opt_recommended + $opt_security + $opt_bad + $CHECK_OTHER` in
        0) # No type options were used.
           CHECK_ALL=1
           ;;
        4) # All type options were used.
           CHECK_ALL=1
           ;;
        *) # Any other combination of types.
           CHECK_ALL=0
           ;;
    esac
}

# ----------------------------- MAIN PROGRAM -----------------------------------

checkOptions $@

if [ $VERBOSE -eq 1 ]; then
    echo "Executing $CMD_PCA --update=never -Hl missing..."
fi

# Parse pca output with awk and print sh variable definitions which
# can then be imported with eval.
#
# Regular Solaris awk does not have a switch/case structure nor
# can it handle a lot of nested 'else if's, so we'll use several
# several search patterns and next statements instead.
#
# Note: Patches are counted using OR logic, not AND. At the end
# of processing P_RS will contain the number of patches with
# R or S or RS, not just patches with RS.
#
eval `$CMD_PCA --update=never -Hl missing | $CMD_AWK '
/^[0-9]/ {
    p_t++;
}
$5 ~ /^---$/ {
    p_o++;
    next;
}
$5 ~ /^R..$/ {
    p_r++;
}
$5 ~ /^.S.$/ {
    p_s++;
}
$5 ~ /^..B$/ {
    p_b++;
}
$5 !~ /^--.$/ {
    p_rs++;
}
$5 !~ /^-.-$/ {
    p_rb++;
}
$5 !~ /^---$/ {
    p_rsb++;
}
$5 !~ /^.--$/ {
    p_sb++;
}
END {
    printf("P_R=%d; P_S=%d; P_B=%d; P_OTHER=%d; P_TOTAL=%d; ", p_r, p_s, p_b, p_o, p_t);
    printf("P_RS=%d; P_RSB=%d; P_RB=%d; P_SB=%d;\n", p_rs, p_rsb, p_rb, p_sb);
}'`

if [ $VERBOSE -eq 1 ]; then
    echo "P_R = $P_R"
    echo "P_S = $P_S"
    echo "P_B = $P_B"
    echo "P_OTHER = $P_OTHER"
    echo "P_TOTAL = $P_TOTAL"
    echo "P_RS  = $P_RS"
    echo "P_RSB = $P_RSB"
    echo "P_RB  = $P_RB"
    echo "P_SB  = $P_SB"
fi

# Figure out the number of patches we are going to compare against
# the warning and critical threshold values.
#
P_AVAILABLE=0
if [ $CHECK_ALL -eq 1 ]; then
    # This one is easy - all or none of the patch types were defined,
    # so we'll use the total number of patches which awk reported to us.
    #
    P_AVAILABLE=$P_TOTAL
else
    # Little shell scripting magic. Using eval and flags set in
    # checkOptions() we can dynamically select correct variable
    # value. E.g. if '-rs' was given in command line options,
    # this will evaluate to 'P_AVAILABLE=$P_RS'.
    #
    eval P_AVAILABLE="\$P_${FLAG_R}${FLAG_S}${FLAG_B}"

    # Don't forget patches with no flags, if '-o' was given.
    #
    if [ $CHECK_OTHER -eq 1 ]; then
        P_AVAILABLE=`expr $P_AVAILABLE + $P_OTHER`
    fi
fi

STATE_TEXT="- $P_AVAILABLE patches available"
PERF_TEXT="A=$P_AVAILABLE R=$P_R S=$P_S B=$P_B O=$P_OTHER T=$P_TOTAL RS=$P_RS RSB=$P_RSB RB=$P_RB SB=$P_SB"

if [ $P_AVAILABLE -ge $PATCH_CRIT_LIMIT ]; then
    echo "PATCH STATUS CRITICAL $STATE_TEXT|$PERF_TEXT"
    exit $STATE_CRITICAL
fi
if [ $P_AVAILABLE -ge $PATCH_WARN_LIMIT ]; then
    echo "PATCH STATUS WARNING $STATE_TEXT|$PERF_TEXT"
    exit $STATE_WARNING
fi
if [ $P_AVAILABLE -lt $PATCH_WARN_LIMIT ]; then
    echo "PATCH STATUS OK $STATE_TEXT|$PERF_TEXT"
    exit $STATE_OK
fi

echo "PATCH STATUS UNKNOWN"
exit $STATE_UNKNOWN
